#
# IDEAS AND COMMENTS
#
# - We should probably cache all App instances on startup to avoid recreating
#   themon each search. Listen to the "changed" signal on the GMenu tree to
#   update
#
# - Matching and scoring algorithm is really lame. Could be more efficient,
#   and calculate the scores better
#
# - Bear in mind when looking at this code that this is *meant* to be simple :-)
#

import sys
from gi.repository import GLib, GObject, Gio
from gi.repository import Dee
from gi.repository import GMenu
from gi.repository import Unity

# These category ids must match the order in which we add them to the lens
CATEGORY_ALL = 0

class App:
	def __init__ (self, uri, icon, cat_id, mime,
	              display_name, comment, dnd_uri,
	              app_info, score = 0):
		self.uri = uri
		self.icon = icon
		self.cat_id = cat_id
		self.mime = mime
		self.display_name = display_name
		self.comment = comment
		self.dnd_uri = dnd_uri
		self.app_info = app_info
		self.score = score
	
	def __lt__ (self, other):
		if other.score :
			return self.score < other.score
		return self.uri < other.uri
	
	def __gt__ (self, other):
		if self.score :
			return self.score > other.score
		return self.uri < other.uri
	
	def __eq__ (self, other):
		return self.uri == other.uri
	
	def __ne__ (self, other):
		return self.uri != other.uri
		
	def __hash__ (self):
		return hash(self.uri)

class Scope (Unity.Scope):

	def __init__ (self):
		Unity.Scope.__init__ (self, dbus_path="/net/launchpad/unitylensbliss/scope/main")
		
		# Listen for changes and requests
		self.connect ("notify::active-search", self._on_search_changed)
		self.connect ("notify::active-global-search", self._on_global_search_changed)
		
		self.apps_tree = GMenu.Tree.new ("unity-lens-bliss.menu", GMenu.TreeFlags.NONE)
		self.apps_tree.load_sync ()
	
	def get_search_string (self):
		search = self.props.active_search
		return search.props.search_string if search else None
	
	def get_global_search_string (self):
		search = self.props.active_global_search
		return search.props.search_string if search else None
	
	def search_finished (self):
		search = self.props.active_search
		if search:
			search.emit("finished")
	
	def global_search_finished (self):
		search = self.props.active_global_search
		if search:
			search.emit("finished")
	
	def _on_search_changed (self, scope, param_spec):		
		search = self.get_search_string()
		results = scope.props.results_model
		
		print "Search changed to: '%s'" % search
		
		self._update_results_model (search, results)
		self.search_finished()
	
	def _on_global_search_changed (self, scope, param_spec):
		search = self.get_global_search_string()
		results = scope.props.global_results_model
		
		print "Global search changed to: '%s'" % search
		
		self._update_results_model (search, results)
		self.global_search_finished()
		
	def _update_results_model (self, search_string, model):
		model.clear ()
		
		# Use a set to collect results to avoid duplicates
		# Apps are compared by app.uri to make this work
		collector = set ()
		self._search_visit_directory (search_string,
		                              collector,
		                              self.apps_tree.get_root_directory ())
		
		results = self._sort_results (search_string, collector)
		
		for app in results:	
			model.append (app.uri,
			              app.icon,
			              app.cat_id,
			              app.mime,
			              app.display_name,
			              app.comment,
			              app.dnd_uri)
		
	
	def _search_visit_directory (self, search_string, collector, tree_dir):
		tree_iter = tree_dir.iter ()
		item_type = tree_iter.next ();
		while (item_type != GMenu.TreeItemType.INVALID):
			if item_type == GMenu.TreeItemType.ENTRY:
				self._search_visit_entry (search_string,
				                          collector,
				                          tree_iter.get_entry ())
			elif item_type == GMenu.TreeItemType.DIRECTORY:
				self._search_visit_directory (search_string,
				                              collector,
				                              tree_iter.get_directory())
			else:
				pass # we skip separators and other stuff
			item_type = tree_iter.next ();
	
	def _search_visit_entry (self, search_string, collector, tree_entry):
		if not search_string:
			self._collect_app (collector, tree_entry)
			return
		
		app_info = tree_entry.get_app_info ()		
		search_string = search_string.lower()
		
		if app_info.get_description () and \
		  search_string in app_info.get_description ().lower():
			self._collect_app (collector, tree_entry, score=1)
		elif app_info.get_display_name () and \
		  search_string in app_info.get_display_name ().lower():
			self._collect_app (collector, tree_entry, score=2)
		elif app_info.get_name () and \
		  search_string in app_info.get_name ().lower():
			self._collect_app (collector, tree_entry, score=2)
		elif app_info.get_executable and \
		  search_string in app_info.get_executable ().lower():
			self._collect_app (collector, tree_entry, score=3)
			
	
	def _collect_app (self, collector, tree_entry, score = 0):
		app_info = tree_entry.get_app_info ()
		app = App ("application://" + tree_entry.get_desktop_file_id (), # uri
		           app_info.get_icon().to_string (),            # string formatted GIcon
		           CATEGORY_ALL,                                # numeric category id
		           "application/x-desktop",                     # mimetype
		           app_info.get_display_name () or "???",       # display name
		           app_info.get_description () or "No description", # comment
		           tree_entry.get_desktop_file_path () or "",   # dnd uri
		           tree_entry.get_app_info (),                  # GAppInfo
		           score)                                       # score
		
		collector.add (app)
	
	def _sort_results (self, search_string, collector):
		return sorted (collector)

